import serial.tools.list_ports
import tkinter
from tkinter import messagebox
import tkinter.ttk
import tkinter.scrolledtext
import teeStd
import consoleLog
import serialCommands
import operationLoop
import measuringLoop
from PIL import Image, ImageTk
import videoRec
import datetime
import multiprocessing

from events import Events


class MyApp(tkinter.Frame):
    def __init__(self, master):
        tkinter.Frame.__init__(self, master)
        self.grid()
        self.ser = serialCommands.SerialCommands()
        self.measure_event_voltage = Events()
        self.measure_event_current = Events()
        self.operation_event_finished = Events()
        self.operation_event_finished.on_change += self.callback_operationFinished
        self.operation_event_start = Events()
        self.operation_event_start.on_change += self.callback_operationStart
        self.measure_event_voltage.on_change += self.callback_measVoltage
        self.measure_event_current.on_change += self.callback_measCurrent
        self.__createWidgets()
        print("sys: start ASR2000 - Control: " + str(datetime.datetime.now()))

    def __createWidgets(self):
        # log output
        self.log = consoleLog.ConsoleLog(window)
        self.log.grid(column=0, row=9, columnspan=5, pady=10, padx=10, sticky='ew')

        #--------------CONNECTION--------------#
        # connection label
        self.lbl_com = tkinter.Label(window, text="Connection", font=("Arial", 12))
        self.lbl_com.grid(column=0, row=1, padx=5, pady=10, sticky='w', columnspan=2)

        # combobox for connections
        self.combo_connection = tkinter.ttk.Combobox(window, state="readonly", width=35)
        self.ports = serial.tools.list_ports.comports()      #show all available ports
        self.combo_connection['values'] = self.ports
        self.combo_connection.grid(column=0, row=2, padx=5, pady=5, sticky='w', columnspan=2)


        # button for connection
        self.btn_connect = tkinter.Button(window, text="connect", command=self.__toggleConnection)
        self.btn_connect.grid(column=2, row=2, padx=5, pady=5, ipadx=10, ipady=5, sticky='w')

        # --------------PARAMETERS--------------#
        # parameters label
        self.lbl_param = tkinter.Label(window, text="Control Settings", font=("Arial", 12))
        self.lbl_param.grid(column=0, row=4, padx=5, pady=10, sticky='w', columnspan=2)

        self.lbl_vType = tkinter.Label(window, text="voltage type", font=("Arial", 10))
        self.lbl_vType.grid(column=0, row=5, padx=5, pady=5, sticky='w')

        self.lbl_vValue = tkinter.Label(window, text="voltage [V]", font=("Arial", 10))
        self.lbl_vValue.grid(column=1, row=5, padx=5, pady=5, sticky='w')

        self.lbl_cValue = tkinter.Label(window, text="max current [A]", font=("Arial", 10))
        self.lbl_cValue.grid(column=2, row=5, padx=5, pady=5, sticky='w')

        self.lbl_freq = tkinter.Label(window, text="frequency [Hz]", font=("Arial", 10))
        self.lbl_freq.grid(column=3, row=5, padx=5, pady=5, sticky='w')

        self.lbl_dur = tkinter.Label(window, text="duration [s]", font=("Arial", 10))
        self.lbl_dur.grid(column=4, row=5, padx=5, pady=5, sticky='w')

        # combobox for voltage setting
        self.combo_vType = tkinter.ttk.Combobox(window, state="readonly", width=10)
        self.combo_vType['values'] = ("PDC (dc50%)", "AC (SIN)", "DC")
        self.combo_vType.bind('<<ComboboxSelected>>', self.callback_combo_vType)
        self.combo_vType.grid(column=0, row=6, padx=5, pady=5, sticky='w')

        # spin box fpr voltage setting
        self.spbox_vValue = tkinter.Spinbox(window, from_=1, to=350, width=10)
        self.spbox_vValue.grid(column=1, row=6, padx=5, pady=5, sticky='w')

        self.spbox_cValue = tkinter.Spinbox(window, from_=0.25, to=1, increment=0.1, width=10)
        self.spbox_cValue.grid(column=2, row=6, padx=5, pady=5, sticky='w')

        self.spbox_freq = tkinter.Spinbox(window, from_=1, to=500, width=10)
        self.spbox_freq.grid(column=3, row=6, padx=5, pady=5, sticky='w')
        self.spbox_freq['state'] = 'disable'

        self.spbox_dur = tkinter.Spinbox(window, from_=1, to=100, width=10)
        self.spbox_dur.grid(column=4, row=6, padx=5, pady=5, sticky='w')

        # label for measuring parameters
        self.lbl_measVoltage = tkinter.Label(window, text="0000", font=("Arial", 10), bg='black', fg='white',
                                        relief='sunken')
        self.lbl_measVoltage.grid(column=1, row=8, padx=3, pady=5, ipadx=20, ipady=5, sticky='w')

        self.lbl_measCurrent = tkinter.Label(window, text="0000", font=("Arial", 10), bg='black', fg='white',
                                        relief='sunken')
        self.lbl_measCurrent.grid(column=2, row=8, padx=3, pady=5, ipadx=20, ipady=5, sticky='w')

        self.btn_start = tkinter.Button(window, text="start", fg='red', command=self.__toggleStart)
        self.btn_start.grid(column=3, row=8, padx=5, pady=20, ipadx=20, ipady=5, sticky='w')
        self.btn_start['state'] = 'disable'

        self.btn_add_camera = tkinter.Button(window, text="open cam", command=self.__toggleAddCam)
        self.btn_add_camera.grid(column=4, row=8, padx=5, pady=20, ipadx=20, ipady=5, sticky='w')

        filename_log = "logs\\log_" + str(datetime.datetime.now()) + ".txt"
        filename_log = filename_log.replace(" ", "_")
        self.filename_log = filename_log.replace(":", "-")
        teeStd.TeeStd(self.log.queue, 'stderr', self.filename_log, "a")
        teeStd.TeeStd(self.log.queue, 'stdout', self.filename_log, "a")

    def callback_combo_vType(self, eventObject):
        if self.combo_vType.current() == 0:
            self.spbox_freq['state'] = 'normal'
        elif self.combo_vType.current() == 1:
            self.spbox_freq['state'] = 'normal'
        elif self.combo_vType.current() == 2:
            self.spbox_freq['state'] = 'disable'

    def callback_measVoltage(self, value_voltage):
        self.lbl_measVoltage.config(text=(value_voltage + " V"))

    def callback_measCurrent(self, value_current):
        self.lbl_measCurrent.config(text=(value_current + " A"))

    def callback_operationStart(self):
        if self.btn_add_camera['text'] == 'close cam':
            self.vid._shock_delivery = True

    def callback_operationFinished(self):
        if self.btn_add_camera['text'] == 'close cam':
            self.vid._shock_delivery = False
        self.btn_start.config(text='start')

    def __toggleConnection(self):
        if self.btn_connect['text'] == 'connect':
            head, sep, tail = self.combo_connection.get().partition(' ')
            self.ser.setConnectionParameters(head)
            self.ser.openConnection()
            self.btn_connect.config(text='disconnect')
            self.btn_start['state'] = 'normal'
        elif self.btn_connect['text'] == 'disconnect':
            self.ser.closeConnection()
            self.btn_connect.config(text='connect')
            self.btn_start['state'] = 'disable'

    def __toggleStart(self):
        if self.btn_start['text'] == 'start':
            self.ser.setVoltageRange(2)     # auto range
            if self.combo_vType.current() == 0:
                self.ser.setVoltageMode(self.combo_vType.current())
                self.ser.setFunctionSQU()
                self.ser.setVoltageSQU(self.spbox_vValue.get())
                self.ser.setMaxCurrent(self.spbox_cValue.get())
                self.ser.setFrequency(self.spbox_freq.get())

            elif self.combo_vType.current() == 1:
                self.ser.setVoltageMode(self.combo_vType.current())
                self.ser.setFunctionSIN()
                self.ser.setVoltageAC(self.spbox_vValue.get())
                self.ser.setMaxCurrent(self.spbox_cValue.get())
                self.ser.setFrequency(self.spbox_freq.get())

            elif self.combo_vType.current() == 2:
                self.ser.setVoltageMode(self.combo_vType.current())
                self.ser.setVoltageDC(self.spbox_vValue.get())
                self.ser.setMaxCurrent(self.spbox_cValue.get())

            self.meas_loop = measuringLoop.MeasuringLoop(self.ser, self.measure_event_voltage,
                                                         self.measure_event_current, self.filename_log)
            self.op_loop = operationLoop.OperationLoop(int(self.spbox_dur.get()), self.ser, self.meas_loop,
                                                       self.operation_event_finished, self.operation_event_start)
            self.op_loop.start()
            self.meas_loop.start()
            self.btn_start.config(text='cancel')

        elif self.btn_start['text'] == 'cancel':
            self.ser.setOutputOff()
            self.meas_loop.setDo(False)
            print("sys: stimulation canceled")
            self.btn_start.config(text='start')

    def __toggleAddCam(self):
        if self.btn_add_camera['text'] == 'open cam':
            self.vid = videoRec.VideoRec(tkinter.Toplevel())
            self.vid.openCamera()
            self.btn_add_camera.config(text='close cam')
            print("sys: open camera gui")
        elif self.btn_add_camera['text'] == 'close cam':
            self.vid.closeWindow()
            self.btn_add_camera.config(text='open cam')
            print("sys: close camera gui")

    def on_closing(self):
        if messagebox.askokcancel("Quit", "Do you want to quit?"):
            self.ser.closeConnection()
            window.destroy()


window = tkinter.Tk()
window.title("synchASR2000 - Control")
window.geometry('740x500')
window.resizable(0, 0)

# icon
ico = Image.open('img\\Arrenberg-Lab-Logo-karminrot-64x41.png')
photo = ImageTk.PhotoImage(ico)
window.wm_iconphoto(False, photo)

app = MyApp(window)

window.protocol("WM_DELETE_WINDOW", app.on_closing)
window.mainloop()
